@@ -12,7 +12,7 @@ from typing import Any, BinaryIO, Generator, Iterable, Iterator, Literal, \
 
 from requests import Session, Response
 
-__version__ = '0.8.0'
+__version__ = '0.9.0'
 
 
 CONFIG: Optional[dict] = None
@@ -203,8 +203,8 @@ class TokenManager(dict):
         super().__init__()
 
     def __missing__(self, token_type) -> str:
-        v = self[token_type] = self.api.query_meta(
-            'tokens', type=token_type)[f'{token_type}token']
+        v = self[token_type] = self.api.meta(
+            'tokens', {'type': token_type})[f'{token_type}token']
         return v
 
 
@@ -349,16 +349,6 @@ class API:
         self.limit = 50
         # action logout returns empty dict on success, thus no return value
 
-    def patrol(self, **params: Any) -> dict:
-        """Patrol a page or revision.
-
-        `token` will be added automatically.
-
-        https://www.mediawiki.org/wiki/API:Patrol
-        """
-        params |= {'action': 'patrol'}
-        return self.post(params)
-
     def _prepare_action(self, /, data: dict):
         if (action := data.get('action')) is None:
             return
@@ -425,10 +415,12 @@ class API:
             yield from self.post_and_continue(data)
 
     def _chunk_value(self, value: Iterable, /):
+        if not value:  # e.g. None or ''
+            return
         if isinstance(value, str):
             value = value.split('|')
         values = iter(value)
-        while chunk := '|'.join(islice(values, self.limit)):
+        while chunk := (*islice(values, self.limit),):
             yield chunk
 
     def _chunk_limited_param(self, data: dict, /):
@@ -443,7 +435,7 @@ class API:
                 continue
             append_violating(param)
             # make sure no data is lost from the param value
-            data[param] = chain((chunk1, chunk2), chunks)
+            data[param] = chain(chunk1, chunk2, chain.from_iterable(chunks))
         if len(violating_params) != 1:
             yield data  # leave it for the API to handle or raise
             return
@@ -487,9 +479,7 @@ class API:
         params['action'] = 'query'
         yield from self.post_and_continue(params)
 
-    def query_list(
-        self, list: str, **params: Any
-    ) -> Generator[dict, None, None]:
+    def list(self, list: str, params: dict) -> Generator[dict, None, None]:
         """Post a list query and yield the results.
 
         https://www.mediawiki.org/wiki/API:Lists
@@ -500,7 +490,7 @@ class API:
             for item in json['query'][list]:
                 yield item
 
-    def query_meta(self, meta, **params: Any) -> dict:
+    def meta(self, meta, params: dict) -> dict:
         """Post a meta query and return the result .
 
         Note: Some meta queries require special handling. Use `self.query()`
@@ -520,9 +510,7 @@ class API:
             assert 'continue' not in json
             return json['query'][meta]
 
-    def query_prop(
-        self, prop: str, **params: Any
-    ) -> Generator[dict, None, None]:
+    def prop(self, prop: str, params: dict) -> Generator[dict, None, None]:
         """Post a prop query, handle batchcomplete, and yield the results.
 
         https://www.mediawiki.org/wiki/API:Properties
